#include <iostream>
#include <fstream>
#include <tuple>
#include <vector>
#include <algorithm>

struct Node {
    int node;
    int w;

    Node(int node, int w) {
        this->node = node;
        this->w = w;
    }
};

void levelList(std::vector<std::vector<Node>> &adjList, std::vector<int> &levels, std::vector<int> &parent,
               std::vector<int> &costParent, int node, int level) {
    levels[node] = level;
    for (auto neighbor: adjList[node]) {
        if (!levels[neighbor.node]) {
            parent[neighbor.node] = node;
            costParent[neighbor.node] = neighbor.w;
            levelList(adjList, levels, parent, costParent, neighbor.node, level + 1);
        }
    }
}

int traversePath(std::vector<int> &levels, std::vector<int> &parent, std::vector<int> &costParent,
                 std::vector<int> &nodeVisits, int x, int y) {
    int pathCost = 0;
    if (levels[x] < levels[y])
        std::swap(x, y);
    while (levels[x] > levels[y]) {
        pathCost += costParent[x];
        nodeVisits[x]++;
        x = parent[x];
    }
    while (x != y) {
        pathCost += costParent[x];
        pathCost += costParent[y];
        nodeVisits[x]++;
        nodeVisits[y]++;
        x = parent[x];
        y = parent[y];
    }

    return pathCost;
}

int main() {
    int N;
    int M, K;

    std::ifstream inputFile("amazon.in");

    inputFile >> N;

    std::vector<std::vector<Node>> adjList(N);

    for (int i = 0; i < N - 1; i++) {
        int x, y, w;
        inputFile >> x >> y >> w;
        adjList[x].emplace_back(y, w);
        adjList[y].emplace_back(x, w);
    }

    std::vector<int> levels(N);
    std::vector<int> parent(N, -1);
    std::vector<int> costParent(N, -1);
    levelList(adjList, levels, parent, costParent, 0, 1);

    inputFile >> M >> K;

    std::vector<int> nodeVisits(N, 0);
    int finalCost = 0;
    for (int i = 0; i < M; i++) {
        int x, y;
        inputFile >> x >> y;
        finalCost += traversePath(levels, parent, costParent, nodeVisits, x, y);
    }

    std::vector<std::pair<int, int>> nodeWithVisits;
    for (int i = 0; i < N; ++i) {
        nodeWithVisits.emplace_back(nodeVisits[i], i);
    }

    std::sort(nodeWithVisits.begin(), nodeWithVisits.end());
    int i = nodeWithVisits.size() - 1;
    while (K > 0 and i >= 0) {
        auto pair = nodeWithVisits[i];
        if (costParent[pair.second] < K) {
            K -= costParent[pair.second];
            finalCost -= costParent[pair.second] * pair.first;
        } else {
            finalCost -= K * pair.first;
            K = 0;
        }
        i -= 1;
    }

    std::ofstream outputFile("amazon.out");
    outputFile << finalCost;

    inputFile.close();
    outputFile.close();

    return 0;
}