#include <iostream>
#include <algorithm>
#include <fstream>
#include <vector>
using namespace std;

ifstream fin("amazon.in");
ofstream fout("amazon.out");

const int MAX_LOG_N = 20;
const int MOD = 666013;

struct AdjacentNode{
    AdjacentNode(int n, int w) : n{n}, w{w}
    {}

    int n;
    int w;
};

struct EdgeWithNumberOfTraversals{
    int w;      // costul muchiei
    int cnt;    // de cate ori apare

    bool operator < (const EdgeWithNumberOfTraversals& other) const
    {
        return cnt > other.cnt;
    }
};

int N;
vector<vector<AdjacentNode>> graph, tree;
vector<int> parent;
vector<int> cost_to_parent;
vector<int> lv;
int M, K;
vector<pair<int, int>> pairs;
int cost;
vector<int> dp[MAX_LOG_N];     // dp[i][j] = stramosul 2^i al lui j
vector<int> sum;

void read_data();
void initial_dfs(int node, int node_parent);
void precompute_lca();
int lca(int x, int y);
void solve();
void dfs_sums(int node);

int main()
{
    read_data();

    cost_to_parent = parent = vector<int>(N);
    tree = vector<vector<AdjacentNode>>(N);
    lv = vector<int>(N);
    initial_dfs(0, -1);

    precompute_lca();
    
    solve();

    fout << cost << '\n';

    fin.close();
    fout.close();
    return 0;
}

void read_data()
{
    fin >> N;

    graph = vector<vector<AdjacentNode>>(N);
    int x, y, w;
    for (int i = 1; i < N; ++i)
    {
        fin >> x >> y >> w;

        graph[x].push_back({y, w});
        graph[y].push_back({x, w});
    }

    fin >> M >> K;
    for (int i = 0; i < M; ++i)
    {
        fin >> x >> y;
        pairs.emplace_back(x, y);
    }
}

void initial_dfs(int node, int node_parent)
{
    parent[node] = node_parent;

    for (const AdjacentNode& adj : graph[node])
    {
        if (adj.n == node_parent)
            continue;
        
        tree[node].emplace_back(adj.n, adj.w);
        lv[adj.n] = lv[node] + 1;
        cost_to_parent[adj.n] = adj.w;
        initial_dfs(adj.n, node);
    }
}

void precompute_lca()
{
    for (int i = 0; i < MAX_LOG_N; ++i)
        dp[i] = vector<int>(N);
    
    // Initializare
    for (int i = 0; i < N; ++i)
        dp[0][i] = parent[i];
    
    // Recurenta
    for (int k = 1; k < MAX_LOG_N; ++k)
        for (int i = 0; i < N; ++i)
        {
            if (dp[k - 1][i] != -1)
                dp[k][i] = dp[k - 1][dp[k - 1][i]];
            else
                dp[k][i] = -1;
        }
}

int lca(int x, int y)
{
    if (lv[x] < lv[y])
        return lca(y, x);
    
    // cout << "lca(" << x << ", " << y << ") = ";

    // lv[x] >= lv[y]
    for (int k = MAX_LOG_N - 1; k >= 0 && lv[x] != lv[y]; --k)
        if (dp[k][x] != -1 && lv[dp[k][x]] >= lv[y])
            x = dp[k][x];
    
    for (int k = MAX_LOG_N - 1; k >= 0 && x != y; --k)
    {
        if (dp[k][x] != -1 && dp[k][x] != dp[k][y])
        {
            x = dp[k][x];
            y = dp[k][y];
        }
    }

    if (x != y)
    {
        // cout << parent[x] << '\n';
        return parent[x];
    }
    else
    {
        // cout << x << '\n';
        return x;
    }
}

void solve()
{
    sum = vector<int>(N);

    int x, y, ancestor;
    for (const pair<int, int>& current_pair : pairs)
    {
        x = current_pair.first;
        y = current_pair.second;
        ancestor = lca(x, y);

        sum[x] += 1;
        sum[y] += 1;
        sum[ancestor] -= 2;
    }

    // for (int i = 0; i < N; ++i)
    // {
    //     cout << "Initial sum(" << i << ") = " << sum[i] << '\n';
    // }

    dfs_sums(0);

    vector<EdgeWithNumberOfTraversals> edges(N - 1);
    for (int i = 1; i < N; ++i)
    {
        edges[i - 1] = {cost_to_parent[i], sum[i]};
    }

    sort(edges.begin(), edges.end());

    for (int i = 0; i < N - 1 && K; ++i)
    {
        if (K >= edges[i].w)
        {
            K -= edges[i].w;
            edges[i].w = 0;
        }
        else
        {
            edges[i].w -= K;
            K = 0;
        }
    }

    cost = 0;
    for (int i = 0; i < N - 1; ++i)
    {
        cost = (1LL * cost + 1LL * edges[i].w * edges[i].cnt) % MOD;
        // cout << i + 1 << ' ' << edges[i].w << ' ' << edges[i].cnt << '\n';
    }
}

void dfs_sums(int node)
{
    for (AdjacentNode adj : tree[node])
    {
        dfs_sums(adj.n);

        sum[node] += sum[adj.n];
    }

    // cout << "Sum(" << node << ") = " << sum[node] << '\n';
}